@@ -12,6 +12,10 @@
 #include <string.h>
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
 #include <srcinst/srcinst.h>
 #include "options.h"
 
@@ -27,7 +31,9 @@ struct option_data action_list[] = {
   { 'u', "uninstall", { 0 }, { 0 }, 0, ". uninstall package PACKAGE_NAME, but do not delete source if available" },
   { 'd', "description", { "STRING", 0 }, { 0, 0 }, 0, ". associate first line of STRING to PACKAGE_NAME short description, and the remaining lines to the long description. If STRING is empty, remove descriptions from the package. Use your shell quotation characters to submit a STRING containing newlines" },
   { 'n', "rename", { "NEW_NAME", 0 }, { 0, 0 }, 0, ". rename existing package PACKAGE_NAME to NEW_NAME" },
-  { 'x', "export", { "FORMAT", "FILENAME", 0 }, { 0, 0, 0 }, 0, ". export package PACKAGE_NAME information to FILENAME. FORMAT can be xml, txt, lst" },
+  { 'X', "export-xml", { "FILENAME", 0 }, { 0, 0 }, 0, ". export package PACKAGE_NAME information (or all packages if not specified) in XML format to FILENAME (use - for stdout)" },
+  { 'T', "export-text", { "FILENAME", 0 }, { 0, 0 }, 0, ". export package PACKAGE_NAME information (or all packages if not specified) in text format to FILENAME (use - for stdout)" },
+  { 'L', "export-list", { "FILENAME", 0 }, { 0, 0 }, 0, ". export package PACKAGE_NAME installed files list to FILENAME (use - for stdout)" },
   { 'k', "pack", { "FILENAME", 0 }, { 0, 0 }, 0, ". build binary tarball from the installed package PACKAGE_NAME, and save it as FILENAME" },
   { '1', "detect-build", { "FILENAME", 0 }, { 0, 0 }, 0, ". scan FILENAME, reporting configure scripts in all subdirectories, and which autoconf version generated them. This can be useful to choose a build subdir, and to run on a new, unknown package to decide whether to install it" },
   { '2', "detect-feats", { "FILENAME", 0 }, { 0, 0 }, 0, ". scan FILENAME, reporting and testing the configure script that will be used (if present), and get the features supported by the generated Makefile. This detection action is useful to run after '-1', coupled with '-D' to choose a particular build subdirectory (and thus configure script/Makefile)" },
