@@ -1,6 +1,6 @@
 /* Source Installer, Copyright (c) 2005-2006 Claudio Fontana
 
- sourceinstall-gtk.c - sourceinstall command line frontend
+ sourceinstall.c - sourceinstall command line frontend
 
  This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -49,16 +49,16 @@
 #include "options.h"
 #include "sourceinstall.h"
 
-char* argz = 0;
+char* program_name = 0;
 struct options o;
 
 void src_error(char* title, char* text, char* o) {
-  fprintf(stderr, "%s: error [%s]: %s %s\n", argz ? argz : "sourceinstall", title, text, o ? o : "");
+  fprintf(stderr, "%s: error [%s]: %s %s\n", program_name ? program_name : "sourceinstall", title, text, o ? o : "");
   exit(1);
 }
 
 void src_warning(char* tit, char* txt, char* o) {
-  fprintf(stderr, "%s: warning [%s]: %s %s\n", argz ? argz : "sourceinstall", tit, txt, o ? o : "");
+  fprintf(stderr, "%s: warning [%s]: %s %s\n", program_name ? program_name : "sourceinstall", tit, txt, o ? o : "");
 }
 
 void src_info(char* title, char* text, char* opt) {
@@ -67,7 +67,7 @@ void src_info(char* title, char* text, char* opt) {
 
 void src_message(char* title, char* text, char* opt) {
   if (!o.quiet) {
-    fprintf(stdout, "%s: [%s]: %s %s\n", argz ? argz : "sourceinstall", title, text, opt ? opt : "");
+    fprintf(stdout, "%s: [%s]: %s %s\n", program_name ? program_name : "sourceinstall", title, text, opt ? opt : "");
   }
 }
 
@@ -83,7 +83,7 @@ int add_package(char* packagename, char* source) {
     return 0;
   }
 
-  rv = install_package_aux(h, source, 1, title);
+  rv = install_package_aux(h, source, 1, 1, title);
   return (srcinst_close_package(h) && rv);
 }
 
@@ -155,7 +155,7 @@ int check_package(char* packagename) {
     
     if ((ssize = srcinst_get_source_size(h)) != (off_t)-1) {
       char* offstr;
-      offstr = srcinst_offtostr(ssize);
+      offstr = srcinst_offtostr(ssize / 1024);
       printf("compressed source size (KB): %s\n", offstr);
       free(offstr);
     }
@@ -172,7 +172,7 @@ int check_package(char* packagename) {
     struct srcinst_string_list* list;
     if ((ssize = srcinst_get_installed_size(h)) != (off_t)-1) {
       char* offstr;
-      offstr = srcinst_offtostr(ssize);
+      offstr = srcinst_offtostr(ssize / 1024);
       printf("installed size (KB): %s\n", offstr);
       free(offstr);
     }
@@ -207,7 +207,11 @@ int check_package(char* packagename) {
   return (srcinst_close_package(h));
 }
 
-int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, char* title) {
+int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, int check_conflicts, char* title) {
+
+  int staged_install;
+  SRCINST_MFEATS mf; int rv = 0;
+  
   if (srcinst_init_build(h, source, o.subdir) != SRCINST_ERR_OK) {
     src_warning(title, "could not initialize build", 0);
     return 0;
@@ -221,12 +225,75 @@ int install_package_aux(SRCINST_PHANDLE h, char* source, int configureq, char* t
     }
   }
   
-  if (srcinst_install(h, o.install, o.source, o.strip, o.force, o.src_compress) != SRCINST_ERR_OK) {
-    src_warning(title, "installation failed", 0);
-    return 0;
+  if (!(mf = srcinst_detect_makefile_features(h))) {
+    src_warning(title, "failed to detect makefile features", 0);
+    goto end_proc;
+  }
+  
+  staged_install = srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) ||
+    srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT);
+ 
+  if (!staged_install) {
+    src_warning(title, "build system does not support staged installs", 0);
+    if (!o.force)
+      goto end_proc;
+  }
+  
+  if (o.source) {
+    if (srcinst_install_source(h, mf, o.src_compress) != SRCINST_ERR_OK) {
+      src_warning(title, "source installation failed", 0);
+      goto end_proc;
+    }
   }
 
-  return 1;
+  if (srcinst_build(h, mf) != SRCINST_ERR_OK) {
+    src_warning(title, "compilation phase failed", 0);
+    goto end_proc;
+  }
+
+  if (o.install) {
+    if (staged_install) {
+      SRCINST_ERR tst_err;
+      if ((tst_err = srcinst_test_install(h, mf, o.strip)) != SRCINST_ERR_OK) {
+	src_warning(title, "test installation reports a fatal error", 0);
+	goto end_proc;
+      }
+
+      if (check_conflicts) {
+	struct srcinst_string_list conflicts;
+
+	if (!srcinst_get_conflict_list(h, &conflicts)) {
+	  src_warning(title, "failed to perform conflict test", 0);
+	  goto end_proc;
+	}
+
+	if (conflicts.count > 0) {
+	  int i;
+	  src_warning(title, "installation would overwrite files claimed by other packages", 0);
+	  for (i = 0; i < conflicts.count; i++) {
+	    src_warning("already claimed", conflicts.strings[i], 0);
+	  }
+	  
+	  if (!o.force) {
+	    srcinst_free_string_list(&conflicts);
+	    goto end_proc;
+	  }
+	}
+	srcinst_free_string_list(&conflicts);
+      }
+    }
+
+    if (srcinst_install_binary(h, mf, o.strip) != SRCINST_ERR_OK) {
+      src_warning(title, "binary installation failed", 0);
+      goto end_proc;
+    }
+  }
+
+  rv = 1;
+  
+ end_proc:
+  srcinst_free_makefile_features(mf);
+  return rv;
 }
 
 int install_package(char* packagename) {
@@ -253,7 +320,7 @@ int install_package(char* packagename) {
     src_warning(title, "source not available for", packagename);
     rv = 0;
   } else {
-    rv = install_package_aux(h, source, configureq, title);
+    rv = install_package_aux(h, source, configureq, 1, title);
   }
 
   return (srcinst_close_package(h) && rv);
@@ -312,6 +379,7 @@ int uninstall_source(char* packagename) {
 int upgrade_package(char* packagename, char* oldname, char* src) {
   SRCINST_PHANDLE h, oldh; int result;
   char* title = "upgrade_package";
+  char* descr, *descr_l;
   
   h = oldh = 0; result = 0;
 
@@ -321,7 +389,7 @@ int upgrade_package(char* packagename, char* oldname, char* src) {
   }
 
   if ((srcinst_get_installed_files(oldh))->count == 0) {
-    src_warning(title, "cannot upgrade non-conforming package", oldname);
+    src_warning(title, "no information about installed files available for package", oldname);
     goto end_proc;
   }
 
@@ -330,25 +398,25 @@ int upgrade_package(char* packagename, char* oldname, char* src) {
     goto end_proc;
   }
 
-  if (!install_package_aux(h, src, 1, title)) {
+  if (!install_package_aux(h, src, 1, 0, title)) {
     src_warning(title, "could not install new package version", 0);
     goto end_proc;
   }
 
+  if ((descr = srcinst_get_descr(oldh))) {
+    srcinst_set_descr(h, descr);
+  }
+  
+  if ((descr_l = srcinst_get_descr_long(oldh))) {
+    srcinst_set_descr_long(h, descr_l);
+  }
+
   result = 1;
 
  end_proc:
   
-  if (h) {
-    result = srcinst_close_package(h) && result;
-  }
-
   if (result) {
-    result = srcinst_upgrade_package(oldh) == SRCINST_ERR_OK;
-  }
-
-  if (oldh) {
-    result = srcinst_close_package(oldh) && result;
+    result = srcinst_upgrade_package(oldh, h) == SRCINST_ERR_OK;
   }
 
   return result;
@@ -403,8 +471,8 @@ int export(char* packagename, char* export_format, char* filename) {
     h = (SRCINST_PHANDLE)0;
 
   } else if (!(h = srcinst_open_package(packagename))) {
-      src_warning(title, "could not open package", packagename);
-      return 0;
+    src_warning(title, "could not open package", packagename);
+    return 0;
   }
 
   if (fmt == SRCINST_EXPORT_LST && 
@@ -633,18 +701,17 @@ int describe_package(char* packagename, char* string) {
   }
 
   if (!string || string[0] == '\0') {
-    rv = SRCINST_ERR_OK == srcinst_set_descr(h, 0) == srcinst_set_descr_long(h, 0);
+    rv = srcinst_set_descr(h, 0) && srcinst_set_descr_long(h, 0);
 
   } else {
     char* mark;
     
     if ((mark = strchr(string, '\n'))) {
       *mark = 0;
-      rv = SRCINST_ERR_OK ==
-	srcinst_set_descr(h, string) == srcinst_set_descr_long(h, mark + 1);
+      rv = srcinst_set_descr(h, string) && srcinst_set_descr_long(h, mark + 1);
 
     } else {
-      rv = SRCINST_ERR_OK == srcinst_set_descr(h, string);
+      rv = srcinst_set_descr(h, string);
     }
   }
 
@@ -715,7 +782,7 @@ int parse_cmdline(int argc, char* argv[]) {
 	      int x;
 	      for (x = 0; data->params[x]; x++) {
 		i++;
-		if (i >= argc || argv[i][0] == '-') {
+		if (i >= argc || argv[i][0] == '-' && argv[i][1] != '\0') {
 		  src_error(title, "missing required option parameter:", data->long_option);
 		}
 		data->values[x] = argv[i];
@@ -771,8 +838,8 @@ int parse_cmdline(int argc, char* argv[]) {
 
 void cmdline_help_usage(void) {
   printf("GNU Source Installer version %s usage:\n\n", PACKAGE_VERSION);
-  printf("%s ACTION [PARAM...] [OPTION [PARAM...]]... [PACKAGE_NAME]\n\n", argz);
-  printf("%s [OPTION [PARAM...]] FILENAME\n", argz);
+  printf("%s ACTION [PARAM...] [OPTION [PARAM...]]... [PACKAGE_NAME]\n\n", program_name);
+  printf("%s [OPTION [PARAM...]] FILENAME\n", program_name);
   printf("    if no action is specified, add FILENAME using defaults.\n");
 }
 
@@ -782,7 +849,7 @@ void cmdline_help(void) {
   cmdline_help_options(action_list);
   printf("\nOTHER OPTIONS:\n");
   cmdline_help_options(option_list);
-  printf("\nSend bug reports to bug-sourceinstall@gnu.org .\n");
+  printf("\nSend bug reports to " PACKAGE_BUGREPORT " .\n");
   exit(0);
 }
 
@@ -909,8 +976,14 @@ int cmdline_options_apply(int selected_action) {
   case 'n':
     return rename_package(o.package, data->values[0]);
 
-  case 'x':
-    return export(o.package, data->values[0], data->values[1]);
+  case 'X':
+    return export(o.package, "xml", data->values[0]);
+
+  case 'T':
+    return export(o.package, "txt", data->values[0]);
+
+  case 'L':
+    return export(o.package, "lst", data->values[0]);
 
   case 'k':
     return export_binary(o.package, data->values[0]);
@@ -935,7 +1008,7 @@ int main(int argc, char* argv[]) {
     src_error(title, "could not initialize libsrcinst", 0);
   }
   atexit(srcinst_fini);
-  argz = (argv[0] ? srcinst_basename(argv[0]) : "");
+  program_name = (argv[0] ? srcinst_basename(argv[0]) : "sourceinstall");
 
   init_options(&o);
   
